function prod = eval_prod(sq_mod,p,f1,f0,ind)

% calculates (f star psi_j1).conj(f star psi_j2) from |f star
% psi_j0|^2
% Inputs :
% - sq_mod : |f star psi_j0|^2
% - p : order of Cauchy wavelets
% - f1 : characteristic frequency of psi_j1
% - f0 : characteristic frequency of psi_j0 ; we assume f1>f0
% - ind : cut-off frequency, determined by the function eval_prod_ref

if (f1>f0)
    
    sq_mod = sq_mod(:).' ;
    N2 = size(sq_mod,2) ;
    sq_mod_ff = fft(sq_mod) ;
    
    % characteristic frequency of psi_j2
    f2 = (2/f0-1/f1)^(-1) ;
    const = (f0^2/(f1*f2))^p ;
    
    ss = [[0:N2/2],[-(N2/2-1):-1]] ;
    
    fourier_mult = exp(p*(-abs(ss-ind)+ind) ...
                       *(-1/f0+1/f2)) ;
    sq_mod_mult = fourier_mult.*sq_mod_ff ;
    prod = ifft(sq_mod_mult)*const ;
    
else
    error('eval_prod: case f1<=f0 not implemented') ;
end