function ind = eval_prod_ref(sq_mod,p,f1,f0,true_prod)

% determines the cut-off frequency for the evaluation of a product
% of components from the squared modulus of one component (as in
% eval_prod.m)
% Inputs :
% - sq_mod : noisy squared modulus
% - p : order of Cauchy wavelets
% - f1 : characteristic frequency of the highest frequency
%        component in the product
% - f0 : characteristic frequency of the squared modulus
% - true_prod : ground true product

sq_mod = sq_mod(:).' ;
N2 = size(sq_mod,2) ;
sq_mod_ff = fft(sq_mod) ;

% characteristic frequency of the lowest frequency component
f2 = (2/f0-1/f1)^(-1) ;
const = (f0^2/(f1*f2))^p ;

s_base = [[0:N2/2],[-(N2/2-1):-1]] ;


if (f1>f0)

    % the research is performed by a kind of dichotomy
    % (significantly faster than the naive way)
    d_old = floor(N2/4) ;
    i_cand = floor(N2/4) ; % best candidate for the cut-off index
                           % so far

    d = d_old ;
    
    while (d>1)
    
        % difference between consecutive candidate indexes
        d = floor(1+d_old/5) ;
        % candidate indexes
        i_test = [max(0,i_cand-d_old):d:min(N2/2,i_cand+d_old)]' ;
        i_test = repmat(i_test,[1,N2]) ;
        ss = repmat(s_base,[size(i_test,1),1]) ;
        fourier_mult = exp(p*(-abs(ss-i_test)+i_test) ...
                           *(-1/f0+1/f2)) ;
        sq_mod_mult = fourier_mult.* ...
            repmat(sq_mod_ff,[size(i_test,1),1]) ;
        % product obtained for each candidate index
        eval_prod = ifft(sq_mod_mult,[],2)*const ;
        
        % difference, for each index, between the estimated product
        % and the ground true product
        diff = eval_prod - repmat(true_prod,[size(i_test,1),1]) ;
        errs = sqrt(sum(abs(diff).^2,2)) ;
        % the new candidate minimizes the error
        [oub,ind] = min(errs) ;

        d_old = d ; i_cand = i_test(ind) ;
    end
    
    ind = i_test(ind) ;
    
else
    error('eval_prod_ref: cas f1<=f0 not implemented') ;
end