function ind = low_frequencies_ref(fphi_noisy,fpsi_true,phi,fs,p)

% determines the ideal cut-off frequency for the evaluation of a
% component of the wavelet transform from the (noisy) low-frequency
% component
% Inputs :
% - fphi_noisy : noisy low-frequency component
% - fphi_true : ground true low-frequency component
% - phi : low-frequency filter
% - fs : frequency of the component to estimate
% - p : order of the wavelets

phi = phi(:) ; fphi_noisy = fphi_noisy(:) ;
fpsi_true = fpsi_true(:).' ;
N = size(phi,1) ;
phi = [phi;zeros(N,1)] ;
N = 2*N ;

ns = [0:N-1] ;
[forg,imax] = max(phi.*(phi<1e-8)) ;

% list of possible cut-off values
ind = [1:imax] ;

phi_ind = repmat(phi(ind),[1,N]) ;
tab_phi = repmat(phi.',[imax,1]) ;
ns = repmat(ns,[imax,1]) ;

tab_psi = (ns/fs).^p.*exp(p*(1-ns/fs)) ;

mult = tab_psi./max(tab_phi,phi_ind) ;
% reconstructed components, depending on the cut-off value
approx_fpsi = ifft(repmat(fft(fphi_noisy).',[imax,1]).*mult,[],2) ;

% error for the different cut-off values
diff = sum(abs(approx_fpsi-repmat(fpsi_true,[imax,1])).^2,2) ;
[forg,ind] = min(diff) ;