clear all ;

disp_bool = 1 ; % enables / disables display

p = 3 ; % order of the Cauchy wavelets
N = 500 ; % size of the signal (must be even)
ln = 1e-3 ; % relative noise on the modulus
opts.Kit = 2000 ; % maximal number of iterations
opts.Kit_gs_rough = 200 ; % number of Gerchberg-Saxton iterations for the
                          % first rough approximation
opts.Kit_gs = 2000 ; % number of Gerchberg-Saxton iterations for the
                     % final refinement
opts.nb_print = 100 ; % display information during the gradient descent

f = create_signal(N,'piecewise_regular') ;

nbf = 8 ; % number of wavelets
[psi,phi,freqs_wav] = cauchy_wavelets(N,p,nbf) ; % generate wavelets

%% generate signal
% you can replace this line by a different definition but the
% signal f must verify \hat f(k)=0 for k>N/2+1
f = f/max(abs(f)) ; % normalization (should not change anything to
                    %                the reconstruction)

% wavelet transform
[fpsi,fphi] = wavelet_transform(f,psi,phi) ;

% add noise
[fpsi_noisy,fphi_noisy] = add_global_noise(abs(fpsi),fphi,ln) ;

[f_rec,obj1,obj2] = reconstruct(fpsi_noisy,fphi_noisy,p,ln,opts) ;

% multiplication by a global phase
al = exp(i*angle(sum(f(:).*conj(f_rec(:))))) ;
f_rec = al*f_rec ;

% wavelet transform of the reconstruction
[fpsi_rec,fphi_rec] = wavelet_transform(f_rec,psi,phi) ;


if disp_bool

    % display the ground true and noisy moduli
    figure (1) ; clf ;
    for s=1:nbf
        subplot(nbf+1,1,s) ; hold on ;
        plot(abs(fpsi(s,:)),'r') ;
        plot(abs(fpsi_noisy(s,:)),'b') ;
        title(sprintf('Ground true / noisy component (%d)',s)) ;
    end
    subplot(nbf+1,1,nbf+1) ; hold on ;
    plot(real(fphi),'r') ;
    plot(real(fphi_noisy)) ;
    title('Ground true / noisy low frequency component') ;

    
    % display the initial and noisy moduli
    figure (2) ; clf ;
    for s=1:nbf
        subplot(nbf+1,1,s) ; hold on ;
        plot(abs(fpsi(s,:)),'r') ;
        plot(abs(fpsi_rec(s,:)),'b') ;
        title(sprintf('Ground true / reconstructed component (%d)',s)) ;
    end
    subplot(nbf+1,1,nbf+1) ; hold on ;
    plot(real(fphi),'r') ;
    plot(real(fphi_rec)) ;
    title('Ground true / reconstructed low frequency component') ;

    % display initial and reconstructed functions
    figure (3) ; clf ; hold on ;
    plot(real(f),'r') ;
    plot(real(f_rec)) ;
    title('Initial function (red) and reconstruction (blue)') ;

    % display objective
    figure (4) ; clf ; hold on ;
    plot(log(obj1(1:end))/log(10),'r') ;
    plot(log(obj2(1:end))/log(10),'m') ;
    obj = obj1+obj2 ;
    plot(log(obj(1:end))/log(10)) ;
    title('Objective function for the gradient descent') ;

end

% print errors
err = [abs(fpsi_rec)-abs(fpsi);fphi_rec-fphi] ;
comp = [abs(fpsi);fphi] ;
fprintf('L2 relative error over the modulus : %f percent\n', ...
        100*norm(err(:),2)/norm(comp(:),2)) ;
fprintf('L2 relative error over f : %f percent\n', ...
        100*norm(f_rec(:)-f(:),2)/norm(f,2)) ;